<?php

namespace App\Http\Controllers\Admin;

use App\Models\Agent;
use App\Models\Photo;
use App\Models\Theme;
use App\Models\Car;
use App\Models\Amenity;
use App\Models\Currency;
use App\Models\Location;
use App\Models\Provider;
use Illuminate\Support\Str;
use App\Models\PropertyType;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Storage;
use App\Http\Requests\CarStoreRequest;
use App\Http\Requests\CarUpdateRequest;

class CarsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $cars = Car::with(['currency','location'])->get();
        return view('admin.cars.index', compact('cars'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $data['currencies'] = Currency::all();
        $data['locations'] = Location::all();
        $data['types'] = PropertyType::all();
        $data['themes'] = Theme::all();
        $data['amenities'] = Amenity::all();
        $data['providers'] = Provider::where('status', 'active')->get();
        $data['agents'] = Agent::where('status', 'active')->get();
        return view('admin.cars.create', $data);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CarStoreRequest $request)
    {
        try {
            DB::beginTransaction();
            $locale = app()->getLocale();
            $car = new Car;
            $car->setTranslation('title', $locale, $request->title);
            $car->setTranslation('seo_title', $locale, ($request->seo_title ? $request->seo_title : $request->title));
            $car->setTranslation('slug', $locale, ($request->slug ? Str::slug($request->slug) : Str::slug($request->title)));
            $car->setTranslation('description', $locale, $request->description);
            $car->setTranslation('seo_description', $locale, $request->seo_description);
            $car->setTranslation('seo_keywords', $locale, $request->seo_keywords);
            $car->price = $request->price;
            $car->status = $request->status;
            $car->currency_id = $request->currency_id;
            $car->duration_unit = $request->unit;
            $car->min_period = $request->min_period;
            $car->max_price = $request->max_price;

            $car->model = $request->model;
            $car->kilometrage = $request->kilometrage;
            $car->carburant = $request->carburant;
            $car->passengers = $request->passengers;
            $car->doors = $request->doors;
            $car->baggages = $request->baggages;
            $car->gear_shift = $request->gear_shift;

            if ($request->has_price_vars) {
                $car->price_by_period = $request->period_prices;
            }
            $car->location_id = $request->location_id;
            $car->type_id = $request->type_id;
            $car->featured = $request->featured ? true : false;
            $car->new = $request->new ? true : false;

            $filename = Car::generate_photo_name(Str::slug($request->title));
            $request->file('featured_image')->storeAs('/', $filename, 'properties');
            $car->featured_image = $filename;

            $car->promo_video = $request->promo_video;
            $car->fetched_url = $request->fetched_url;
            $car->provider_id = $request->provider_id;
            $car->agent_id = $request->agent_id;
            $car->save();
            // themes + amenities
            $car->themes()->sync($request->themes);
            $car->amenities()->sync($request->amenities);
            DB::commit();
            return response()->json(['msg' => 'success', 'url' => route('admin.cars.index')]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $data['car'] = Car::with(['themes:id', 'amenities:id'])->findOrFail($id);
        $data['car_themes'] = $data['car']->themes->pluck('id')->toArray();
        $data['car_amenities'] = $data['car']->amenities->pluck('id')->toArray();
        $data['currencies'] = Currency::all();
        $data['locations'] = Location::all();
        $data['types'] = PropertyType::all();
        $data['themes'] = Theme::all();
        $data['amenities'] = Amenity::all();
        $data['providers'] = Provider::where('status', 'active')->get();
        $data['agents'] = Agent::where('status', 'active')->get();
        return view('admin.cars.edit', $data);
    }

    public function photos($car)
    {
        $data['car'] = Car::findOrFail($car);
        $data['photos'] = $data['car']->photos;
        return view('admin.cars.photos', $data);
    }

    public function save_photos(Request $request, $car)
    {
        $car = Car::findOrFail($car);
        $old_photos = $car->photos;
        if($request->selected_photos && count($request->selected_photos) != count($old_photos)){
            foreach($old_photos as $old){
                if(!in_array($old->id,$request->selected_photos)){
                    if (Storage::disk('properties')->exists('/' . $old->filename)) {
                        Storage::disk('properties')->delete('/' . $old->filename);
                    }
                    $old->delete();
                }
            }
        }
        if ($request->hasFile('pictures')) {
            $allowedfileExtension = ['jpeg', 'jpg', 'png', 'webp'];
            $files = $request->file('pictures');
            $photos = [];
            foreach ($files as $file) {
                $filename = Car::generate_photo_name(Str::slug($car->title) .'-'. uniqid());
                $extension = $file->getClientOriginalExtension();
                $check = in_array($extension, $allowedfileExtension);
                if ($check) {
                    $file->storeAs('/', $filename, 'properties');
                    $photos[] = new Photo(['filename' => $filename]);
                }
            }
            $car->photos()->saveMany($photos);
        }
        return response()->json(['msg' => 'success']);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(CarUpdateRequest $request, $id)
    {
        try {
            DB::beginTransaction();
            $locale = app()->getLocale();
            $car = Car::findOrFail($id);
            $car->setTranslation('title', $locale, $request->title);
            $car->setTranslation('seo_title', $locale, ($request->seo_title ? $request->seo_title : $request->title));
            $car->setTranslation('slug', $locale, ($request->slug ? Str::slug($request->slug) : Str::slug($request->title)));
            $car->setTranslation('description', $locale, $request->description);
            $car->setTranslation('seo_description', $locale, $request->seo_description);
            $car->setTranslation('seo_keywords', $locale, $request->seo_keywords);
            $car->price = $request->price;
            $car->status = $request->status;
            $car->currency_id = $request->currency_id;
            $car->duration_unit = $request->unit;
            $car->min_period = $request->min_period;
            $car->max_price = $request->max_price;

            $car->model = $request->model;
            $car->kilometrage = $request->kilometrage;
            $car->carburant = $request->carburant;
            $car->passengers = $request->passengers;
            $car->doors = $request->doors;
            $car->baggages = $request->baggages;
            $car->gear_shift = $request->gear_shift;

            if ($request->has_price_vars) {
                $car->price_by_period = $request->period_prices;
            } else {
                $car->price_by_period = null;
            }
            $car->location_id = $request->location_id;
            $car->type_id = $request->type_id;
            $car->featured = $request->featured ? true : false;
            $car->new = $request->new ? true : false;

            if ($request->hasFile('featured_image')) {
                $oldFilename = $car->featured_image;
                $filename = Car::generate_photo_name(Str::slug($request->title));
                $request->file('featured_image')->storeAs('/', $filename, 'properties');
                $car->featured_image = $filename;
                if (Storage::disk('properties')->exists('/' . $oldFilename)) {
                    Storage::disk('properties')->delete('/' . $oldFilename);
                }
            }

            $car->promo_video = $request->promo_video;
            $car->fetched_url = $request->fetched_url;
            $car->provider_id = $request->provider_id;
            $car->agent_id = $request->agent_id;
            $car->save();
            // themes + amenities
            $car->themes()->sync($request->themes);
            $car->amenities()->sync($request->amenities);
            DB::commit();
            return response()->json(['msg' => 'success', 'url' => route('admin.cars.index')]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            DB::beginTransaction();
            $car = Car::findOrFail($id);
            if (Storage::disk('properties')->exists('/' . $car->featured_image)) {
                Storage::disk('properties')->delete('/' . $car->featured_image);
            }
            // get photos and delete them
            $photos = $car->photos;
            $ids = [];
            foreach($photos as $photo){
                if (Storage::disk('properties')->exists('/' . $photo->filename)) {
                    Storage::disk('properties')->delete('/' . $photo->filename);
                }
                $ids[] = $photo->id;
            }
            Photo::destroy($ids);
            $car->delete();
            // themes + amenities
            $car->themes()->sync([]);
            $car->amenities()->sync([]);
            DB::commit();
            return response()->json(['msg' => 'success', 'url' => route('admin.cars.index')]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }
}
